const App = @import("./App.zig");

const coral = @import("coral");

const gfx = @import("./gfx.zig");

const msg = @import("./msg.zig");

const std = @import("std");

pub const Axis = struct {
	keys: ?[2]gfx.Input.Key = null,
};

pub const Mapping = struct {
	keys_pressed: ScancodeSet = ScancodeSet.initEmpty(),
	keys_held: ScancodeSet = ScancodeSet.initEmpty(),

	const ScancodeSet = std.bit_set.StaticBitSet(512);

	pub fn axis_strength(self: Mapping, axis: Axis) f32 {
		if (axis.keys) |keys| {
			const key_down, const key_up = keys;
			const is_key_down_held = self.keys_held.isSet(key_down.scancode());
			const is_key_up_held = self.keys_held.isSet(key_up.scancode());

			if (is_key_down_held or is_key_up_held) {
				return
					@as(f32, @floatFromInt(@intFromBool(is_key_up_held))) -
					@as(f32, @floatFromInt(@intFromBool(is_key_down_held)));
			}
		}

		return 0;
	}
};

pub fn setup(world: *coral.World, events: App.Events) std.mem.Allocator.Error!void {
	try world.set_resource(.none, Mapping{});

	try world.on_event(events.pre_update, coral.system_fn(update), .{
		.label = "update act",
	});
}

pub fn update(inputs: msg.Receive(gfx.Input), mapping: coral.Write(Mapping)) void {
	mapping.res.keys_pressed = Mapping.ScancodeSet.initEmpty();

	for (inputs.messages()) |message| {
		switch (message) {
			.key_down => |key| {
				mapping.res.keys_pressed.set(key.scancode());
				mapping.res.keys_held.set(key.scancode());
			},

			.key_up => |key| {
				mapping.res.keys_held.unset(key.scancode());
			},
		}
	}
}
