const coral = @import("coral");

const std = @import("std");

pub fn Chain(comptime Command: type, comptime clone_command: ?Clone(Command)) type {
	return struct {
		swap_lists: [2]CommandList,
		swap_state: u1 = 0,

		const CommandList = List(Command, clone_command);

		const Self = @This();

		pub fn deinit(self: *Self) void {
			for (&self.swap_lists) |*list| {
				list.deinit();
			}

			self.* = undefined;
		}

		pub fn init(allocator: std.mem.Allocator) Self {
			return .{
				.swap_lists = .{CommandList.init(allocator), CommandList.init(allocator)},
			};
		}

		pub fn pending(self: *Self) *CommandList {
			return &self.swap_lists[self.swap_state];
		}

		pub fn submitted(self: *Self) *CommandList {
			return &self.swap_lists[self.swap_state ^ 1];
		}

		pub fn swap(self: *Self) void {
			self.swap_state ^= 1;
		}
	};
}

pub fn Clone(comptime Command: type) type {
	return fn (Command, *std.heap.ArenaAllocator) std.mem.Allocator.Error!Command;
}

pub fn List(comptime Command: type, comptime clone_command: ?Clone(Command)) type {
	return struct {
		arena: std.heap.ArenaAllocator,
		stack: coral.stack.Sequential(Command),

		const Self = @This();

		pub fn append(self: *Self, command: Command) std.mem.Allocator.Error!void {
			return self.stack.push_grow(if (clone_command) |clone| try clone(command, &self.arena) else command);
		}

		pub fn clear(self: *Self) void {
			self.stack.clear();

			if (!self.arena.reset(.retain_capacity)) {
				std.log.warn("failed to reset the buffer of a gfx queue with retained capacity", .{});
			}
		}

		pub fn deinit(self: *Self) void {
			self.arena.deinit();
			self.stack.deinit();

			self.* = undefined;
		}

		pub fn init(allocator: std.mem.Allocator) Self {
			return .{
				.arena = std.heap.ArenaAllocator.init(allocator),
				.stack = .{.allocator = allocator},
			};
		}
	};
}

