const coral = @import("coral");

const gfx = @import("../gfx.zig");

const std = @import("std");

pub fn bmp_file_desc(arena: *std.heap.ArenaAllocator, path: []const u8) gfx.Assets.Format.Error!gfx.Desc {
	const header = try coral.files.bundle.read_little(path, 0, extern struct {
		type: [2]u8 align (1),
		file_size: u32 align (1),
		reserved: [2]u16 align (1),
		image_offset: u32 align (1),
		header_size: u32 align (1),
		pixel_width: i32 align (1),
		pixel_height: i32 align (1),
		color_planes: u16 align (1),
		bits_per_pixel: u16 align (1),
		compression_method: u32 align (1),
		image_size: u32 align(1),
		pixels_per_meter_x: i32 align (1),
		pixels_per_meter_y: i32 align (1),
		palette_colors_used: u32 align (1),
		important_colors_used: u32 align (1),
	}) orelse {
		return error.FormatUnsupported;
	};

	if (!std.mem.eql(u8, &header.type, "BM")) {
		return error.FormatUnsupported;
	}

	const pixel_width = std.math.cast(u16, header.pixel_width) orelse {
		return error.FormatUnsupported;
	};

	const pixels = try arena.allocator().alloc(coral.io.Byte, header.image_size);
	const bytes_per_pixel = header.bits_per_pixel / coral.io.bits_per_byte;
	const alignment = 4;
	const byte_stride = pixel_width * bytes_per_pixel;
	const padded_byte_stride = alignment * @divTrunc((byte_stride + alignment - 1), alignment);
	const byte_padding = coral.scalars.sub(padded_byte_stride, byte_stride) orelse 0;
	var buffer_offset: usize = 0;
	var file_offset = @as(usize, header.image_offset);

	switch (header.bits_per_pixel) {
		32 => {
			while (buffer_offset < pixels.len) {
				const line = pixels[buffer_offset .. buffer_offset + byte_stride];

				if (try coral.files.bundle.read_bytes(path, file_offset, line) != byte_stride) {
					return error.FormatUnsupported;
				}

				for (0 .. pixel_width) |i| {
					const line_offset = i * 4;
					const pixel = line[line_offset .. line_offset + 4];

					std.mem.swap(u8, &pixel[0], &pixel[2]);
				}

				file_offset += line.len + byte_padding;
				buffer_offset += padded_byte_stride;
			}
		},

		else => return error.FormatUnsupported,
	}

	return .{
		.texture = .{
			.width = pixel_width,
			.data = pixels,
			.format = .rgba8,
			.access = .static,
		}
	};
}
