@header const Vec2 = @Vector(2, f32)
@ctype vec2 Vec2

@vs vs
in vec2 mesh_xy;
in vec2 mesh_uv;

in vec2 instance_xbasis;
in vec2 instance_ybasis;
in vec2 instance_origin;
in vec4 instance_tint;
in float instance_depth;
in vec4 instance_rect;

uniform Screen {
	vec2 screen_size;
};

out vec4 color;
out vec2 uv;

void main() {
	// Calculate the world position of the vertex
	const vec2 world_position = instance_origin + mesh_xy.x * instance_xbasis + mesh_xy.y * instance_ybasis;

	// Convert world position to normalized device coordinates (NDC)
	// Assuming the screen coordinates range from (0, 0) to (screen_size.x, screen_size.y)
	const vec2 ndc_position = (vec2(world_position.x, -world_position.y) / screen_size) * 2.0 - vec2(1.0, -1.0);

	// Set the position of the vertex in clip space
	gl_Position = vec4(ndc_position, instance_depth, 1.0);
	color = instance_tint;

	// Calculate the width and height from left, top, right, bottom configuration
	const vec2 rect_pos = instance_rect.xy; // left, top
	const vec2 rect_size = instance_rect.zw - instance_rect.xy; // right - left, bottom - top

	// Calculate the adjusted UV coordinates using the instance_rect
	uv = rect_pos + (mesh_uv * rect_size);
}
@end

@fs fs
uniform texture2D tex;
uniform sampler smp;

in vec4 color;
in vec2 uv;

out vec4 texel;

void main() {
	texel = texture(sampler2D(tex, smp), uv) * color;

	if (texel.a == 0) {
		discard;
	}
}
@end

@program draw_2d vs fs
